@@ -141,10 +141,10 @@ void generate_units_tree_struct() {
 					}
 					if(!b) {
 						tree_struct cat;
+						cat.parent = item;
 						item->items.push_back(cat);
 						it = item->items.end();
 						--it;
-						it->parent = item;
 						item = &*it;
 						item->item = cat_sub;
 					}
@@ -205,10 +205,10 @@ void generate_variables_tree_struct() {
 					}
 					if(!b) {
 						tree_struct cat;
+						cat.parent = item;
 						item->items.push_back(cat);
 						it = item->items.end();
 						--it;
-						it->parent = item;
 						item = &*it;
 						item->item = cat_sub;
 					}
@@ -270,10 +270,10 @@ void generate_functions_tree_struct() {
 					}
 					if(!b) {
 						tree_struct cat;
+						cat.parent = item;
 						item->items.push_back(cat);
 						it = item->items.end();
 						--it;
-						it->parent = item;
 						item = &*it;
 						item->item = cat_sub;
 					}
